@@ -1,8 +1,8 @@
 # Command line interface for the executor package.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: November 14, 2015
-# URL: https://executor.readthedocs.org
+# Last Change: June 24, 2017
+# URL: https://executor.readthedocs.io
 #
 # TODO Expose a clean way to interrupt the fudge factor of other processes.
 # TODO Properly document command timeout / lock-timeout / TERM-timeout / KILL-timeout.
@@ -58,11 +58,11 @@ Supported options:
 
   -v, --verbose
 
-    Make more noise than usual (increase logging verbosity).
+    Increase logging verbosity (can be repeated).
 
   -q, --quiet
 
-    Make less noise than usual (decrease logging verbosity).
+    Decrease logging verbosity (can be repeated).
 
   -h, --help
 
@@ -189,10 +189,9 @@ def apply_fudge_factor(fudge_factor):
     :param fudge_factor: The maximum number of seconds to sleep (a number).
 
     Previous implementations of the fudge factor interrupt used UNIX signals
-    (specifically :data:`~signal.SIGUSR1`) but the use of this signal turned
-    out to be sensitive to awkward race conditions and it wasn't very cross
-    platform, so now the creation of a regular file is used to interrupt the
-    fudge factor.
+    (specifically ``SIGUSR1``) but the use of this signal turned out to be
+    sensitive to awkward race conditions and it wasn't very cross platform, so
+    now the creation of a regular file is used to interrupt the fudge factor.
     """
     if fudge_factor:
         timer = Timer()
