@@ -1,8 +1,8 @@
 # Programmer friendly subprocess wrapper.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: November 8, 2015
-# URL: https://executor.readthedocs.org
+# Last Change: December 18, 2016
+# URL: https://executor.readthedocs.io
 
 """
 Support for concurrent external command execution.
@@ -10,7 +10,10 @@ Support for concurrent external command execution.
 The :mod:`executor.concurrent` module defines the :class:`CommandPool` class
 which makes it easy to prepare a large number of external commands, group them
 together in a pool, start executing a configurable number of external commands
-simultaneously and wait for all external commands to finish.
+simultaneously and wait for all external commands to finish. For fine grained
+concurrency control please refer to the :attr:`~.ExternalCommand.dependencies`
+and :attr:`~.ExternalCommand.group_by` properties of the
+:class:`.ExternalCommand` class.
 """
 
 # Standard library modules.
@@ -62,9 +65,27 @@ class CommandPool(PropertyManager):
         This is a positive integer number. It defaults to the return value of
         :func:`multiprocessing.cpu_count()` (which may not make much sense if
         your commands are I/O bound instead of CPU bound).
+
+        Setting :attr:`concurrency` to one is a supported use case intended to
+        make it easier for users of the :mod:`executor.concurrent` module to
+        reuse the code they've built on top of command pools (if only for
+        debugging, but there are lots of use cases :-).
         """
         return multiprocessing.cpu_count()
 
+    @mutable_property
+    def delay_checks(self):
+        """
+        Whether to postpone raising an exception until all commands have run (a boolean).
+
+        If this option is :data:`True` (not the default) and a command with
+        :attr:`.check` set to :data:`True` fails the command pool's execution
+        is not aborted, instead all commands will be allowed to run. After all
+        commands have finished a :exc:`CommandPoolFailed` exception will be
+        raised that tells you which command(s) failed.
+        """
+        return False
+
     @mutable_property
     def logger(self):
         """
@@ -95,19 +116,6 @@ class CommandPool(PropertyManager):
         .. _tail -f: https://en.wikipedia.org/wiki/Tail_(Unix)#File_monitoring
         """
 
-    @mutable_property
-    def delay_checks(self):
-        """
-        Whether to postpone raising an exception until all commands have run (a boolean).
-
-        If this option is :data:`True` (not the default) and a command with
-        :attr:`.check` set to :data:`True` fails the command pool's execution
-        is not aborted, instead all commands will be allowed to run. After all
-        commands have finished a :exc:`CommandPoolFailed` exception will be
-        raised that tells you which command(s) failed.
-        """
-        return False
-
     @property
     def is_finished(self):
         """:data:`True` if all commands in the pool have finished, :data:`False` otherwise."""
@@ -133,22 +141,47 @@ class CommandPool(PropertyManager):
         """The number of currently running commands in the pool (an integer)."""
         return sum(cmd.is_running for id, cmd in self.commands)
 
+    @property
+    def running_groups(self):
+        """
+        A set of running command groups.
+
+        The value of :attr:`running_groups` is a :class:`set` with the
+        :attr:`~.ExternalCommand.group_by` values of all currently running
+        commands (:data:`None` is never included in the set).
+        """
+        return set(
+            cmd.group_by for id, cmd in self.commands
+            if cmd.is_running and cmd.group_by is not None
+        )
+
     @property
     def results(self):
         """
         A mapping of identifiers to external command objects.
 
         This is a dictionary with external command identifiers as keys (refer
-        to :func:`add()`) and :class:`~executor.ExternalCommand` objects as
-        values. The :class:`~executor.ExternalCommand` objects provide access
-        to the return codes and/or output of the finished commands.
+        to :func:`add()`) and :class:`.ExternalCommand` objects as values. The
+        :class:`.ExternalCommand` objects provide access to the return codes
+        and/or output of the finished commands.
         """
         return dict(self.commands)
 
+    @mutable_property
+    def spinner(self):
+        """
+        Whether to show an animated spinner or not (a boolean or :data:`None`).
+
+        The value of :attr:`spinner` defaults to :data:`None` which means a
+        spinner is shown when we're connected to a terminal but hidden when
+        we're not connected to a terminal.
+        """
+        return None
+
     @property
     def unexpected_failures(self):
         """
-        A list of :class:`ExternalCommand` objects that *failed unexpectedly*.
+        A list of :class:`~executor.ExternalCommand` objects that *failed unexpectedly*.
 
         The resulting list includes only commands where :attr:`.check` and
         :attr:`.failed` are both :data:`True`.
@@ -160,7 +193,7 @@ class CommandPool(PropertyManager):
         Add an external command to the pool of commands.
 
         :param command: The external command to add to the pool (an
-                        :class:`~executor.ExternalCommand` object).
+                        :class:`.ExternalCommand` object).
         :param identifier: A unique identifier for the external command (any
                            value). When this parameter is not provided the
                            identifier is set to the number of commands in the
@@ -169,19 +202,35 @@ class CommandPool(PropertyManager):
                          (the identifier with ``.log`` appended) in case
                          :attr:`logs_directory` is set.
 
-        The :attr:`~executor.ExternalCommand.async` property of command objects
-        is automatically set to :data:`True` when they're added to a
-        :class:`CommandPool`. If you really want the commands to execute with a
-        concurrency of one (1) then you can set :attr:`concurrency` to one
-        (I'm not sure why you'd want to do that though :-).
+        When a command is added to a command pool the following options are
+        changed automatically:
+
+        - The :attr:`~executor.ExternalCommand.async` property is set to
+          :data:`True`. If you want the commands to execute with a concurrency
+          of one then you should set :attr:`concurrency` to one.
+
+        - The :attr:`~executor.ExternalCommand.tty` property is set to
+          :data:`False` when :attr:`concurrency` is higher than one because
+          interaction with multiple concurrent subprocesses in a single
+          terminal is prone to serious miscommunication (when multiple
+          subprocesses present an interactive prompt at the same time and the
+          user tries to answer one of the prompts it will be impossible to tell
+          which of the subprocesses will receive the user's reply).
         """
+        # Configure the command to run asynchronously.
         command.async = True
+        # Configure the command to run without a controlling terminal?
+        if self.concurrency > 1:
+            command.tty = False
+        # Override the command's default logger?
         if command.logger == parent_logger:
             command.logger = self.logger
+        # Pick a default identifier for the command?
         if identifier is None:
             identifier = len(self.commands) + 1
+        # Configure logging of command output?
         if self.logs_directory:
-            if not log_file:
+            if log_file is None:
                 log_file = '%s.log' % identifier
             pathname = os.path.join(self.logs_directory, log_file)
             directory = os.path.dirname(pathname)
@@ -190,6 +239,7 @@ class CommandPool(PropertyManager):
             handle = open(pathname, 'ab')
             command.stdout_file = handle
             command.stderr_file = handle
+        # Add the command to the pool.
         self.commands.append((identifier, command))
 
     def run(self):
@@ -207,6 +257,10 @@ class CommandPool(PropertyManager):
         If you're writing code where you want to own the main loop then
         consider calling :func:`spawn()` and :func:`collect()` directly instead
         of using :func:`run()`.
+
+        When :attr:`concurrency` is set to one, specific care is taken to make
+        sure that the callbacks configured by :attr:`.start_event` and
+        :attr:`.finish_event` are called in the expected (intuitive) order.
         """
         # Start spawning processes to execute the commands.
         timer = Timer()
@@ -214,10 +268,18 @@ class CommandPool(PropertyManager):
                      pluralize(self.num_commands, "command"),
                      self.concurrency)
         try:
-            with Spinner(timer=timer) as spinner:
+            with Spinner(interactive=self.spinner, timer=timer) as spinner:
+                num_started = 0
+                num_collected = 0
                 while not self.is_finished:
-                    self.spawn()
-                    self.collect()
+                    # When concurrency is set to one (I know, initially it
+                    # sounds like a silly use case, bear with me) I want the
+                    # start_event and finish_event callbacks of external
+                    # commands to fire in the right order. The following
+                    # conditional is intended to accomplish this goal.
+                    if self.concurrency > (num_started - num_collected):
+                        num_started += self.spawn()
+                    num_collected += self.collect()
                     spinner.step(label=format(
                         "Waiting for %i/%i %s",
                         self.num_commands - self.num_finished, self.num_commands,
@@ -245,13 +307,35 @@ class CommandPool(PropertyManager):
 
         :returns: The number of external commands that were spawned by this
                   invocation of :func:`spawn()` (an integer).
+
+        The commands to start are picked according to three criteria:
+
+        1. The command's :attr:`~.ExternalCommand.was_started` property is
+           :data:`False`.
+        2. The command's :attr:`~.ExternalCommand.group_by` value is not
+           present in :attr:`running_groups`.
+        3. The :attr:`~.ExternalCommand.is_finished` properties of all of the
+           command's :attr:`~.ExternalCommand.dependencies` are :data:`True`.
         """
         num_started = 0
-        todo = [cmd for id, cmd in self.commands if not cmd.was_started]
-        while todo and self.num_running < self.concurrency:
-            cmd = todo.pop(0)
-            cmd.start()
-            num_started += 1
+        limit = self.concurrency - self.num_running
+        if limit > 0:
+            running_groups = self.running_groups
+            for id, cmd in self.commands:
+                # Skip commands that have already been started.
+                if not cmd.was_started:
+                    # If command groups are being used we'll only
+                    # allow one running command per command group.
+                    if cmd.group_by not in running_groups:
+                        # If a command has any dependencies we won't allow it
+                        # to start until all of its dependencies have finished.
+                        if all(dependency.is_finished for dependency in cmd.dependencies):
+                            cmd.start()
+                            num_started += 1
+                            if cmd.group_by is not None:
+                                running_groups.add(cmd.group_by)
+                            if num_started == limit:
+                                break
         if num_started > 0:
             logger.debug("Spawned %s ..", pluralize(num_started, "external command"))
         return num_started
@@ -310,7 +394,8 @@ class CommandPool(PropertyManager):
         If :func:`terminate()` successfully terminates commands, you then call
         :func:`collect()` and the :attr:`.check` property of a terminated
         command is :data:`True` you will get an exception because terminated
-        commands (by definition) report a nonzero :attr:`.returncode`.
+        commands (by definition) report a nonzero
+        :attr:`~executor.ExternalCommand.returncode`.
         """
         num_terminated = 0
         for identifier, command in self.commands:
